<?php

/**
 * @file
 * The admin functions for the module.
 *
 * Handles the administration interface callbacks.
 */

/**
 * Callback for the main Block Inject page.
 */
function block_inject_list() {
  $result = block_inject_get_regions();
  $header = array(
    array('data' => t('Inject Regions')),
    array('data' => t('Operations'), 'colspan' => 3),
  );
  $rows = array();
  foreach ($result as $row) {
    $tablerow = array(
      array('data' => check_plain($row->region) . ' | ' . check_plain($row->node_name)),
      array('data' => l(t('Edit'), 'admin/structure/block-inject/' . $row->id . '/edit')),
      array('data' => l(t('Delete'), 'admin/structure/block-inject/' . $row->id . '/delete')),
    );
    $rows[] = $tablerow;
  }
  if (!$rows) {
    $rows[] = array(array('data' => t('No inject regions available.'), 'colspan' => 4));
  }
  $build = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#attributes' => array('id' => 'block_inject'),
  );

  return $build;
}

/**
 * Callback for the Block Inject Exceptions page.
 */
function block_inject_exceptions_list() {
  $result = block_inject_get_exceptions();
  $header = array(
    array('data' => t('Excepted Nodes')),
    array('data' => t('Node type')),
    array('data' => t('Edit')),
  );
  $rows = array();
  
  foreach ($result as $row) {
    $node_type = node_type_load($row->type);
    $tablerow = array(
      array('data' => l(check_plain($row->title), 'node/' . $row->nid)),
      array('data' => check_plain($node_type->name)),
      array('data' => l(t('Edit'), 'node/' . $row->nid . '/edit')),
    );
    $rows[] = $tablerow;
  }
  if (!$rows) {
    $rows[] = array(array('data' => t('No exceptions available.'), 'colspan' => 4));
  }
  $build = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#attributes' => array('id' => 'block_inject_exceptions'),
  );

  return $build;
}

/**
 * Callback form to add new region.
 */
function block_inject_add_inject_form($form, &$form_state) {
  $form['block_inject_region_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Region name'),
    '#description' => t('What is the name of the region you would like to inject?'),
    '#required' => TRUE,
  );
  // Return available node types to have as options for the select form element.
  $node_types = block_inject_get_node_types();
  $form['block_inject_content_type'] = array(
    '#type' => 'select',
    '#title' => t('Available Content Type(s)'),
    '#description' => t('Which content type would you like this region to be injected in the middle of?'),
    '#options' => $node_types,
    '#multiple' => TRUE,
  );
  $form['block_inject_region_submit'] = array(
    '#type' => 'submit',
    '#value' => t('Create region'),
  );

  return $form;
}

/**
 * Submit form that adds the new region to the database.
 */
function block_inject_add_inject_form_submit($form, &$form_state) {
  // Sanitize the region name submited by the user.
  $region = $form_state['input']['block_inject_region_name'];
  $name_test = block_inject_check_region_name($region);
  if ($name_test == TRUE) {
    if (isset($form_state['input']['block_inject_content_type'])) {
      // If set, retrieve the machine name of the selected node typs.
      $node_type_array = $form_state['input']['block_inject_content_type'];
      // Retrieve the names of the selected node types.
      $node_type_name_array = array_flip(array_intersect(array_flip($form_state['complete form']['block_inject_content_type']['#options']), $node_type_array));
      // Implode the node type arrays.
      $node_type_name = check_plain(implode(', ', $node_type_name_array));
      $node_type = check_plain(implode(', ', $node_type_array));
    }
    else {
      $node_type = '';
      $node_type_name = '';
    }
    // Check if region name was provided and add to database.
    if ($region) {
      block_inject_add_region($region, $node_type, $node_type_name);
      // Redirect back to main Block Inject page.
      $form_state['redirect'] = 'admin/structure/block-inject';
      // Rebuild data and clear cache.
      block_inject_rebuild_data();
      drupal_set_message(t('Your region has been added.'), 'status');
    }
    else {
      drupal_set_message(t('There was an error'), 'error');
    }
  }
  else {
    drupal_set_message(t('Duplicate region name. Please choose another one!'), 'error');
  }
}

/**
 * Callback form to edit a region.
 *
 * @param int $id
 *   Id of the region to be edited.
 */
function block_inject_edit_form($form, &$form_state, $id) {
  // Get the region by the ID that came through the the URL.
  $region = block_inject_get_region_by_id($id);
  // Pass the region id through the form_state for the submit function.
  $form_state['block_inject_region_id'] = $id;
  $form['block_inject_region_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Region name'),
    '#description' => t('What is the name of the region you would like to inject?'),
    '#required' => TRUE,
    '#default_value' => $region->region,
  );
  // Get the available node types to be selected.
  $available_node_types = block_inject_get_node_types();
  // Get the node types selected by this region.
  $sel_node_type_machine_array = explode(', ', $region->node_type);
  $sel_node_type_name_array = explode(', ', $region->node_name);
  $selected_node_types = array_combine($sel_node_type_machine_array, $sel_node_type_name_array);
  // Add to the available node types the ones selected by this region.
  $node_types_options = array_merge($available_node_types, $selected_node_types);
  $form['block_inject_content_type'] = array(
    '#type' => 'select',
    '#title' => t('Available Content Type(s)'),
    '#description' => t('Which content type would you like this region to be injected in the middle of?'),
    '#options' => $node_types_options,
    '#multiple' => TRUE,
    '#default_value' => array_flip($selected_node_types),
  );
  $form['block_inject_region_submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save changes'),
  );

  return $form;
}

/**
 * Submit form to edit a region.
 */
function block_inject_edit_form_submit($form, &$form_state) {
  // Sanitize the region name submited by the user.
  $region = check_plain($form_state['input']['block_inject_region_name']);
  // Run the name through the function to check if it is already in use.
  $name_test = block_inject_check_region_name($region);
  if (($name_test == TRUE) || ($region == $form['block_inject_region_name']['#default_value'])) {
    if (isset($form_state['input']['block_inject_content_type'])) {
      // If set, retrieve the machine name of the selected node typs.
      $node_type_array = $form_state['input']['block_inject_content_type'];
      // Retrieve the names of the selected node types.
      $node_type_name_array = array_flip(array_intersect(array_flip($form_state['complete form']['block_inject_content_type']['#options']), $node_type_array));
      // Implode the node type arrays.
      $node_type_name = check_plain(implode(', ', $node_type_name_array));
      $node_type = check_plain(implode(', ', $node_type_array));
    }
    else {
      $node_type = '';
      $node_type_name = '';
    }
    // Check if region name was provided and add to database.
    if ($region) {
      $id = $form_state['block_inject_region_id'];
      block_inject_update_region($id, $region, $node_type, $node_type_name);
      // Redirect back to main Block Inject page.
      $form_state['redirect'] = 'admin/structure/block-inject';
      block_inject_rebuild_data();
      drupal_set_message(t('Your region has been updated.'), 'status');
    }
    else {
      drupal_set_message(t('There was an error'), 'error');
    }
  }
  else {
    drupal_set_message(t('Duplicate region name. Please choose another one!'), 'error');
  }
}

/**
 * Callback function for confirming the inject region delete.
 *
 * @param int $region_id
 *   Id of the region to be deleted.
 */
function block_inject_delete_confirm($form, &$form_state, $region_id) {
  $region = block_inject_get_region_by_id($region_id);
  // Pass the region through the form_state for the submit function.
  $form_state['block_inject_region'] = $region;
  return confirm_form($form, t('Are you sure you want to delete the block inject region %title ?', array('%title' => $region->region)), 'admin/structure/block-inject', 'The blocks that are assigned to this region will be reassigned. This action cannot be undone!', t('Delete'), t('Cancel'));
}

/**
 * Callback function to delete the inject region from the database.
 */
function block_inject_delete_confirm_submit($form, &$form_state) {
  $region = $form_state['block_inject_region'];
  block_inject_remove_region($region->id);
  block_inject_remove_exception('bi_id', $region->id);
  drupal_set_message(t('The region %title has been removed.', array('%title' => $region->region)));
  $form_state['redirect'] = 'admin/structure/block-inject';
}
